# Copyright 2020 Red Hat, Inc. All rights reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
# http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

import abc


class ImageBase(object, metaclass=abc.ABCMeta):

    @abc.abstractmethod
    def get_image_id_by_tag(self, image_tag, image_owner=None):
        """Get image ID by image tag and owner.

        :param image_tag: image tag
        :param image_owner: optional image owner
        :raises: ImageGetException if no images found with given tag
        :return: image id
        """
